"""Example: wrap-up card"""

# pylint: disable=C0413

import sys

from requests import Response

sys.path.append("../../")
import adaptive_cards.card_types as types
from adaptive_cards.actions import ActionToggleVisibility, TargetElement
from adaptive_cards.card import AdaptiveCard
from adaptive_cards.client import TeamsClient
from adaptive_cards.containers import Column, ColumnSet, Container, ContainerTypes
from adaptive_cards.elements import Image, TextBlock
from adaptive_cards.validation import CardValidator, CardValidatorFactory
from result import Result, is_ok

containers: list[ContainerTypes] = []

icon_source: str = "https://icons8.com/icon/vNXFqyQtOSbb/launch"
icon_url: str = "https://img.icons8.com/3d-fluency/94/launched-rocket.png"

header_column_set: ColumnSet = ColumnSet(
    columns=[
        Column(
            items=[
                TextBlock(text="Your Daily Wrap-Up", size=types.FontSize.EXTRA_LARGE)
            ],
            width="stretch",
        ),
        Column(items=[Image(url=icon_url, width="40px")], rtl=True, width="auto"),
    ]
)
containers.append(
    Container(
        items=[header_column_set], style=types.ContainerStyle.EMPHASIS, bleed=True
    )
)

containers.append(
    Container(
        items=[
            TextBlock(
                text="**Some numbers for you**",
                size=types.FontSize.MEDIUM,
            ),
            ColumnSet(
                columns=[
                    Column(
                        items=[
                            TextBlock(text="_Total_"),
                            TextBlock(text="_Done by you_"),
                            TextBlock(text="_Done by other teams_"),
                            TextBlock(text="_Still open_"),
                            TextBlock(text="_Closed_"),
                        ]
                    ),
                    Column(
                        items=[
                            TextBlock(text="5"),
                            TextBlock(text="4"),
                            TextBlock(text="3"),
                            TextBlock(text="6"),
                            TextBlock(text="1"),
                        ],
                        spacing=types.Spacing.MEDIUM,
                        rtl=True,
                    ),
                ],
                separator=True,
            ),
        ],
        spacing=types.Spacing.MEDIUM,
    )
)

containers.append(
    Container(
        items=[
            TextBlock(
                text="**Detailed Results**",
                size=types.FontSize.MEDIUM,
            ),
        ],
        separator=True,
        spacing=types.Spacing.EXTRA_LARGE,
    )
)

sample_column_set: ColumnSet = ColumnSet(
    columns=[
        Column(items=[TextBlock(text="12312")]),
        Column(items=[TextBlock(text="done", color=types.Colors.GOOD)]),
        Column(items=[TextBlock(text="abc")]),
        Column(
            items=[
                Image(
                    url="https://adaptivecards.io/content/down.png",
                    width="20px",
                    horizontal_alignment=types.HorizontalAlignment.RIGHT,
                )
            ],
            select_action=ActionToggleVisibility(
                title="More",
                target_elements=[
                    TargetElement(
                        element_id="toggle-me",
                    )
                ],
            ),
        ),
    ]
)

containers.append(
    Container(
        items=[
            Container(
                items=[
                    ColumnSet(
                        columns=[
                            Column(items=[TextBlock(text="**Number**")]),
                            Column(items=[TextBlock(text="**Status**")]),
                            Column(items=[TextBlock(text="**Topic**")]),
                            Column(items=[TextBlock(text="")]),
                        ],
                        id="headline",
                    ),
                ],
                style=types.ContainerStyle.EMPHASIS,
                bleed=True,
            ),
            Container(items=[sample_column_set]),
            Container(
                items=[
                    TextBlock(
                        text="_Here you gonna find more information about the whole topic_",
                        id="toggle-me",
                        is_visible=False,
                        is_subtle=True,
                        wrap=True,
                    )
                ]
            ),
        ],
    )
)

containers.append(
    Container(
        items=[
            TextBlock(
                text=f"Icon used from: {icon_source}",
                size=types.FontSize.SMALL,
                horizontal_alignment=types.HorizontalAlignment.CENTER,
                is_subtle=True,
            )
        ]
    )
)

# Build card
card = AdaptiveCard.new().version("1.5").add_items(containers).create()


# Validate card
card_validator: CardValidator = CardValidatorFactory.create_validator_microsoft_teams()
result: Result = card_validator.validate(card)

print(f"Validation was successful: {is_ok(result)}")

# send card
webhook_url: str = "YOUR-URL"
client: TeamsClient = TeamsClient(webhook_url)
response: Response = client.send(card)



from adaptive_cards.card import AdaptiveCard
from adaptive_cards.inputs import InputText
from result import is_ok, is_err
from adaptive_cards.actions import ActionSubmit


class TestAdaptiveCard:
    def test_create_card(self):
        card = AdaptiveCard.new().version("1.2").create()
        assert card.type == "AdaptiveCard"
        assert card.version == "1.2"

    def test_add_item(self):
        input_text: InputText = InputText(id="test", label="test")
        card: AdaptiveCard = AdaptiveCard.new().add_item(input_text).create()
        assert card.body is not None
        assert input_text in card.body

    def test_update_item_success(self):
        input_text: InputText = InputText(id="id", label="test")
        card = AdaptiveCard.new().add_item(input_text).version("1.3").create()

        input_text_updated: InputText = InputText(id="id", label="updated")
        card_updated = (
            AdaptiveCard.new().add_item(input_text_updated).version("1.3").create()
        )

        result = card.update_item("id", label="updated")

        assert is_ok(result)
        assert card_updated == card

    def test_update_item_not_found(self):
        input_text: InputText = InputText(id="id", label="test")
        card = AdaptiveCard.new().add_item(input_text).version("1.3").create()

        input_text_updated: InputText = InputText(id="id", label="updated")
        card_updated = (
            AdaptiveCard.new().add_item(input_text_updated).version("1.3").create()
        )

        result = card.update_item("wrong-id", label="updated")

        assert is_err(result)
        assert card_updated != card

    def test_update_item_field_not_found(self):
        input_text: InputText = InputText(id="id", label="test")
        card = AdaptiveCard.new().add_item(input_text).version("1.3").create()

        input_text_updated: InputText = InputText(id="id", label="updated")
        card_updated = (
            AdaptiveCard.new().add_item(input_text_updated).version("1.3").create()
        )

        result = card.update_item("id", wrong_field="updated")

        assert is_err(result)
        assert card_updated != card

    def test_update_item_wrong_value_type(self):
        input_text: InputText = InputText(id="id", label="test")
        card = AdaptiveCard.new().add_item(input_text).version("1.3").create()

        input_text_updated: InputText = InputText(id="id", label="updated")
        card_updated = (
            AdaptiveCard.new().add_item(input_text_updated).version("1.3").create()
        )

        result = card.update_item("id", label=1)

        assert is_err(result)
        assert card_updated != card

    def test_add_action(self):
        action_submit: ActionSubmit = ActionSubmit(id="action1", title="Submit")
        card: AdaptiveCard = AdaptiveCard.new().add_action(action_submit).create()
        assert card.actions is not None
        assert action_submit in card.actions

    def test_add_actions(self):
        action_submit1: ActionSubmit = ActionSubmit(id="action1", title="Submit1")
        action_submit2: ActionSubmit = ActionSubmit(id="action2", title="Submit2")
        card: AdaptiveCard = (
            AdaptiveCard.new().add_actions([action_submit1, action_submit2]).create()
        )
        assert card.actions is not None
        assert action_submit1 in card.actions
        assert action_submit2 in card.actions

    def test_update_action_success(self):
        action_submit: ActionSubmit = ActionSubmit(id="action1", title="Submit")
        card = AdaptiveCard.new().add_action(action_submit).version("1.3").create()

        result = card.update_action("action1", title="Updated Submit")

        assert is_ok(result)
        assert card._actions["action1"].title == "Updated Submit"

    def test_update_action_element_not_found(self):
        action_submit: ActionSubmit = ActionSubmit(id="action1", title="Submit")
        card = AdaptiveCard.new().add_action(action_submit).version("1.3").create()

        result = card.update_action("wrong-id", title="Updated Submit")

        assert is_err(result)

    def test_update_action_field_not_found(self):
        action_submit: ActionSubmit = ActionSubmit(id="action1", title="Submit")
        card = AdaptiveCard.new().add_action(action_submit).version("1.3").create()

        result = card.update_action("action1", wrong_field="Updated Submit")

        assert is_err(result)

    def test_update_action_wrong_value_type(self):
        action_submit: ActionSubmit = ActionSubmit(id="action1", title="Submit")
        card = AdaptiveCard.new().add_action(action_submit).version("1.3").create()

        result = card.update_action("action1", title=123)

        assert is_err(result)



"""Tests for validation module"""

from adaptive_cards.card import AdaptiveCard
from adaptive_cards.validation import (
    CardValidator,
    CardValidatorFactory,
    ValidationFailure,
)
from adaptive_cards.elements import TextBlock
from adaptive_cards.card_types import FontType
from adaptive_cards import utils
from result import is_ok, is_err
from pydantic import BaseModel, Field


class TestCardValidation:
    def test_card_validator_ms_teams_validate_success(self):
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )
        card: AdaptiveCard = (
            AdaptiveCard.new().add_item(TextBlock(text="Test Card")).create()
        )
        assert is_ok(validator.validate(card))
        assert len(validator.details()) == 0

    def test_validate_failure_empty_body(self):
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )
        card: AdaptiveCard = AdaptiveCard.new().create()
        assert is_err(validator.validate(card))
        assert len(validator.details()) == 1
        assert validator.details()[0].failure == ValidationFailure.EMPTY_CARD

    def test_validate_failure_invalid_field_version(self):
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )

        # should fail as font_type is available only card schemas > 1.2
        text_block = TextBlock(text="Test Card", font_type=FontType.MONOSPACE)
        card: AdaptiveCard = (
            AdaptiveCard.new().version("1.0").add_item(text_block).create()
        )
        assert is_err(validator.validate(card, debug=True))
        assert len(validator.details()) == 1
        assert validator.details()[0].failure == ValidationFailure.INVALID_FIELD_VERSION

    def test_validate_failure_invalid_schema(self):
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )

        class InvalidClass(BaseModel):
            """Invalid class"""

            some_field: int | None = Field(
                default=None, json_schema_extra=utils.get_metadata("1.0")
            )

        card: AdaptiveCard = (
            AdaptiveCard.new()
            .version("1.0")
            .add_item(InvalidClass(some_field=1))  # type: ignore
            .create()
        )
        assert is_err(validator.validate(card))
        assert len(validator.details()) == 1
        assert validator.details()[0].failure == ValidationFailure.INVALID_SCHEMA

    def test_validate_failure_size_limit_exceeded(self):
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )

        card: AdaptiveCard = (
            AdaptiveCard.new()
            .version("1.0")
            .add_items([TextBlock(text="TestCard") for i in range(730)])
            .create()
        )

        assert validator.card_size(card) < 28
        assert is_ok(validator.validate(card))

        card = (
            AdaptiveCard.new()
            .version("1.0")
            .add_items([TextBlock(text="TestCard") for i in range(740)])
            .create()
        )
        assert validator.card_size(card) > 28
        assert is_err(validator.validate(card))
        assert len(validator.details()) == 1
        assert validator.details()[0].failure == ValidationFailure.SIZE_LIMIT_EXCEEDED



from adaptive_cards.card import AdaptiveCard
from adaptive_cards.validation import CardValidatorFactory, CardValidator, Finding
from result import Result, is_ok
from adaptive_cards.elements import TextBlock
import adaptive_cards.card_types as types

text_block: TextBlock = TextBlock(
    text="It's your second card",
    color=types.Colors.ACCENT,
    size=types.FontSize.EXTRA_LARGE,
    horizontal_alignment=types.HorizontalAlignment.CENTER,
)

version: str = "1.4"
card: AdaptiveCard = (
    AdaptiveCard.new().version(version).add_items([text_block]).create()
)

validator: CardValidator = CardValidatorFactory.create_validator_microsoft_teams()
result: Result[None, str] = validator.validate(card)

print(f"Validation was successful: {is_ok(result)}")

card_size: float = validator.card_size(card)
print(card_size)

details: list[Finding] = validator.details()
assert len(details) == 0



import pytest
from unittest.mock import patch, Mock
from http import HTTPStatus
from adaptive_cards.card import AdaptiveCard
from adaptive_cards.client import TeamsClient
from adaptive_cards.elements import TextBlock

# filepath: src/adaptive_cards/test_client.py


class TestTeamsClient:
    @patch("adaptive_cards.client.requests.post")
    def test_send_single_card_success(self, mock_post):
        mock_response = Mock()
        mock_response.status_code = HTTPStatus.OK
        mock_post.return_value = mock_response

        client = TeamsClient("http://example.com/webhook")
        card = AdaptiveCard.new().add_item(TextBlock(text="Test Card")).create()
        response = client.send(card)

        assert response.status_code == HTTPStatus.OK
        mock_post.assert_called_once()

    @patch("adaptive_cards.client.requests.post")
    def test_send_multiple_cards_success(self, mock_post):
        mock_response = Mock()
        mock_response.status_code = HTTPStatus.ACCEPTED
        mock_post.return_value = mock_response

        client = TeamsClient("http://example.com/webhook")
        card1 = AdaptiveCard.new().add_item(TextBlock(text="Test Card")).create()
        card2 = AdaptiveCard.new().add_item(TextBlock(text="Test Card")).create()
        response = client.send(card1, card2)

        assert response.status_code == HTTPStatus.ACCEPTED
        mock_post.assert_called_once()

    def test_send_no_webhook_url(self):
        client = TeamsClient("")
        card = AdaptiveCard.new().add_item(TextBlock(text="Test Card")).create()

        with pytest.raises(ValueError, match="No webhook URL provided."):
            client.send(card)

    def test_send_no_cards_provided(self):
        client = TeamsClient("http://example.com/webhook")

        with pytest.raises(ValueError, match="No cards provided."):
            client.send()

    @patch("adaptive_cards.client.requests.post")
    def test_send_failed_request(self, mock_post):
        mock_response = Mock()
        mock_response.status_code = HTTPStatus.BAD_REQUEST
        mock_response.text = "Bad Request"
        mock_post.return_value = mock_response

        client = TeamsClient("http://example.com/webhook")
        card = AdaptiveCard.new().add_item(TextBlock(text="Test Card")).create()

        with pytest.raises(RuntimeError, match="Failed to send card: 400, Bad Request"):
            client.send(card)



"""Example: simple card"""

import adaptive_cards.card_types as types
from adaptive_cards.actions import ActionOpenUrl
from adaptive_cards.card import AdaptiveCard
from adaptive_cards.elements import TextBlock
from result import Result, is_ok

text_block: TextBlock = TextBlock(
    id="text-id",
    text="Initial text",
)

action_open_url: ActionOpenUrl = ActionOpenUrl(
    id="action-id", url="any-url", title="title"
)

# build card
version: str = "1.4"
card: AdaptiveCard = (
    AdaptiveCard.new().add_item(text_block).add_action(action_open_url).create()
)

# update card item
update_result: Result[None, str] = card.update_item(
    id="text-id",
    text="New text",
    horizontal_alignment=types.HorizontalAlignment.CENTER,
    font_type=types.FontType.MONOSPACE,
)
assert is_ok(update_result)
print(card)

# update card action
update_result = card.update_action(id="action-id", url="new-url")
assert is_ok(update_result)
print(card)



"""Implementations for all adaptive card action types"""

from __future__ import annotations
from typing import Any, Optional, Union

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel

import adaptive_cards.card_types as ct
from adaptive_cards import utils


class Action(BaseModel):
    # pylint: disable=too-many-instance-attributes
    """
    Represents an action that can be performed.

    Attributes:
        title: An optional string representing the title of the action.
        icon_url: An optional string representing the URL of the icon associated with the action.
        id: An optional string representing the ID of the action.
        style: An optional ActionStyle enum value representing the style of the action.
        fallback: An optional fallback ActionTypes object representing the fallback action to be
        performed.
        tooltip: An optional string representing the tooltip text for the action.
        is_enabled: An optional boolean indicating whether the action is enabled or disabled.
        mode: An optional ActionMode enum value representing the mode of the action.
        requires: An optional dictionary mapping string keys to string values representing the
        requirements for the action.
    """

    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    title: Optional[str] = Field(
        default=None,
        json_schema_extra=utils.get_metadata("1.0"),
    )
    icon_url: Optional[str] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.1")
    )
    id: Optional[str] = Field(default=None, json_schema_extra=utils.get_metadata("1.0"))  # pylint: disable=C0103
    style: Optional[ct.ActionStyle] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.2")
    )
    fallback: Optional["ActionTypes"] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.2")
    )
    tooltip: Optional[str] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.5")
    )
    is_enabled: Optional[bool] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.5")
    )
    mode: Optional[ct.ActionMode] = Field(
        default=ct.ActionMode.PRIMARY, json_schema_extra=utils.get_metadata("1.5")
    )
    requires: Optional[dict[str, str]] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.2")
    )


class ActionOpenUrl(Action):
    """
    Represents an action to open a URL.

    Inherits from Action

    Attributes:
        url: The URL to be opened.
        type: The type of the action. Default is "Action.OpenUrl".
    """

    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    url: str = Field(json_schema_extra=utils.get_metadata("1.0"))
    type: str = Field(
        default="Action.OpenUrl",
        json_schema_extra=utils.get_metadata("1.0"),
        frozen=True,
    )


class ActionSubmit(Action):
    """
    Represents an action to submit data.

    Inherits from Action.

    Attributes:
        type: The type of the action. Default is "Action.Submit".
        data: Optional data associated with the action.
        associated_inputs: Optional associated inputs for the action.
    """

    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    type: str = Field(
        default="Action.Submit",
        json_schema_extra=utils.get_metadata("1.0"),
        frozen=True,
    )
    data: Optional[str | Any] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.0")
    )
    associated_inputs: Optional[ct.AssociatedInputs] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.3")
    )


class ActionShowCard(Action):
    """
    Represents an action to show a card.

    Inherits from Action.

    Attributes:
        type: The type of the action. Default is "Action.ShowCard".
        card: Optional card to show.
    """

    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    type: str = Field(
        default="Action.ShowCard",
        json_schema_extra=utils.get_metadata("1.0"),
        frozen=True,
    )
    card: Optional[Any] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.0")
    )


class TargetElement(BaseModel):
    """
    Represents a target element.

    Attributes:
        element_id: The ID of the target element.
        is_visible: Optional flag indicating the visibility of the target element.
    """

    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    element_id: str = Field(json_schema_extra=utils.get_metadata("1.0"))
    is_visible: Optional[bool] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.0")
    )


class ActionToggleVisibility(Action):
    """
    Represents an action that toggles the visibility of target elements.

    Inherits from Action.

    Attributes:
        target_elements: A list of TargetElement objects representing the target elements to toggle.
        type: The type of the action, set to "Action.ToggleVisibility".
    """

    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    target_elements: list[TargetElement] = Field(
        json_schema_extra=utils.get_metadata("1.2")
    )
    type: str = Field(
        default="Action.ToggleVisibility",
        json_schema_extra=utils.get_metadata("1.2"),
        frozen=True,
    )


class ActionExecute(Action):
    """
    Represents an action that executes a command or performs an action.

    Inherits from Action.

    Attributes:
        type: The type of the action, set to "Action.ShowCard".
        verb: An optional string representing the verb of the action.
        data: An optional string or Any type representing additional data associated
        with the action.
        associated_inputs: An optional AssociatedInputs object representing associated
        inputs for the action.
    """

    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    type: str = Field(
        default="Action.ShowCard",
        json_schema_extra=utils.get_metadata("1.4"),
        frozen=True,
    )
    verb: Optional[str] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.4")
    )
    data: Optional[str | Any] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.4")
    )
    associated_inputs: Optional[ct.AssociatedInputs] = Field(
        default=None, json_schema_extra=utils.get_metadata("1.4")
    )


ActionTypes = Union[
    ActionOpenUrl,
    ActionSubmit,
    ActionShowCard,
    ActionToggleVisibility,
    ActionExecute,
]

SelectAction = Union[
    ActionExecute,
    ActionOpenUrl,
    ActionSubmit,
    ActionToggleVisibility,
]

Action.model_rebuild()
